@@ -1,4 +1,5 @@
 # Copyright (C) 1999, 2000 Jay Beale
+# Copyright (C) 2001, 2002 Hewlett Packard Company
 # Licensed under the GNU General Public License
 
 package Bastille::FilePermissions;
@@ -23,7 +24,7 @@ use Bastille::API;
 &GeneralPerms;
 #&Activatelibsafe;
 &SUIDAudit;
-&WritableDirAudit;
+&WriteableDirAudit;
 
 sub GeneralPerms {
 
@@ -38,7 +39,7 @@ sub GeneralPerms {
 # from a TrinityOS script.
 # 
 
-   my $distro=$GLOBAL_DISTRO;
+   my $distro=&GetDistro;
 
    if (&getGlobalConfig("FilePermissions","generalperms_1_1") eq "Y") {
 
@@ -244,48 +245,6 @@ sub GeneralPerms {
 	   &ErrorLog("Didn't carry out general permissions modifications as they are very specific to Red Hat / Mandrake\n");
        }
    }
-   if (&getGlobalConfig("FilePermissions", "generalperms_1_2_mandrake")) {
-       &ActionLog("# sub GeneralPerms -- 1.2_mandrake");
-
-       my $security_level = &getGlobalConfig("FilePermissions","generalperms_1_2_mandrake");
-
-       # Check the security level
-       my $security_level_is_ok = 0;
-       my $lvl;
-       foreach $lvl (0,1,2,3,4,5) {
-	   if ($lvl eq $security_level) {
-	       $security_level_is_ok =1;
-	   }
-       }
-
-       # Change permissions according to Mandrake's level files
-       # BUG!  This doesn't respect our B_PREFIX at all!
-       unless ($security_level_is_ok) { $security_level = 3; }
-       my $return = open PERMFILE, "/usr/share/Bastille/mandrake_perm.$security_level";
-       if ($return) {
-	   while (my $line = <PERMFILE>) {
-	       if ($line = /^(.*)\s+([^\.]+).([^\.]+)\s+(\d+)/) {
-		   my $file=$1;
-		   my $owner=$2;
-		   my $group=$3;
-		   my $mode=$4;
-
-		   # BUG: may get wrong value if $GLOBAL_PREFIX is not ''
-		   my $uid = (getpwnam($owner))[2];
-		   my $gid = (getgrnam($group))[2];
-		   
-		   &B_chown($uid,$file);
-		   &B_chgrp($gid,$file);
-		   &B_chmod_if_exists($mode,$file);
-	       }
-	   }
-       }
-       else {
-	   &ErrorLog("Failed to open permfile /usr/share/Bastille/mandrake_perm.$security_level\nFile permissions not changed\n");
-       }
-
-   }
-
 }
 
 
@@ -331,8 +290,7 @@ sub SUIDAudit {
 
    if (&getGlobalConfig("FilePermissions","suidping") eq "Y") {
        &B_remove_suid(&getGlobal('BIN',"ping"));
-       # this appears to be for Debian but is never defined?
-       #&B_remove_suid(&getGlobal('BIN',"ping6"));
+       &B_remove_suid(&getGlobal('BIN',"ping6"));
    }
     
 #/sbin/pwdb_chkpwd
@@ -340,16 +298,26 @@ sub SUIDAudit {
    if (&getGlobalConfig("FilePermissions","suiddump") eq "Y") {
        &B_remove_suid(&getGlobal('BIN',"dump"));
        &B_remove_suid(&getGlobal('BIN',"restore"));
+       if (&GetDistro =~ /^OSX/) {
+	   &B_remove_suid(&getGlobal('BIN',"rdump"));
+	   &B_remove_suid(&getGlobal('BIN',"rrestore"));
+       }
    }
 
    if (&getGlobalConfig("FilePermissions","suidcard") eq "Y") {
       &B_remove_suid(&getGlobal('BIN',"cardctl"));
    }
     
-#/usr/X11R6/bin/Xwrapper
+   if (&getGlobalConfig("FilePermissions","suidXwrapper") eq "Y") {
+      &B_remove_suid(&getGlobal('BIN',"Xwrapper"));
+   }
     
    if (&getGlobalConfig("FilePermissions","suidat") eq "Y") {
        &B_remove_suid(&getGlobal('BIN',"at"));
+       if (&GetDistro =~ /^OSX/) {
+	   &B_remove_suid(&getGlobal('BIN',"atq"));
+	   &B_remove_suid(&getGlobal('BIN',"atrm"));
+       }
    }
 
 #/usr/bin/crontab
@@ -376,7 +344,7 @@ sub SUIDAudit {
        &B_remove_suid(&getGlobal('BIN',"lpq"));
        &B_remove_suid(&getGlobal('BIN',"lpr"));
        &B_remove_suid(&getGlobal('BIN',"lprm"));
-       if ($GLOBAL_DISTRO =~ "^HP-UX") {
+       if (&GetDistro =~ "^HP-UX") {
 	   &B_remove_suid(&getGlobal('BIN',"lpalt"));
        }	   
    }
@@ -390,7 +358,8 @@ sub SUIDAudit {
        &B_chmod_if_exists(000,&getGlobal('BIN',"rcp"));
        &B_chmod_if_exists(000,&getGlobal('BIN',"rlogin"));
        &B_chmod_if_exists(000,&getGlobal('BIN',"rsh"));
-       if ( $GLOBAL_DISTRO =~ "^HP-UX") {
+       if ( &GetDistro =~ "^HP-UX") {
+	   print "DEBUG: This is an HP system?\n\n";
 	   &B_chmod_if_exists(000,&getGlobal('BIN',"rdist"));
 	   &B_chmod_if_exists(000,&getGlobal('BIN',"rexec"));
        }   
@@ -407,30 +376,16 @@ sub SUIDAudit {
 
    if (&getGlobalConfig("FilePermissions","suidtrace") eq "Y") {
        &B_remove_suid( &getGlobal('BIN',"traceroute"));
-       # for Debian, but not defined?
-       #&B_remove_suid( &getGlobal('BIN',"traceroute6"));
+       &B_remove_suid( &getGlobal('BIN',"traceroute6"));
    }
 
 #/usr/sbin/userhelper
 #/usr/libexec/pt_chown
-   
-# The following "To Do" list logic applies to all suid files, but right now is asked only on HP-UX
-   if (&getGlobalConfig("FilePermissions","suid_general") eq "Y") {
-       my $suid_audit_text = 
-	"Do an SUID audit to disable unnecessary SUID programs.  :\n    " .
-        "The command is: \"find / -perm -4000\".  This will find all \n" .
-        "files on your system with the SUID bit set.  Be sure not to \n" .
-        "remove the SUID bit on \"su\" or \"login\", because you will be \n" .
-        "unable to log back in as root to fix things.  Note that disabling \n" .
-        "SUID status on files can break your system in sometimes \n" .
-        "unpredictable ways, so be sure to test your changes on a \n" .
-        "non-production system first. \n \n \n";
-       &B_TODO($suid_audit_text);
-   }
-}    
+}
+
 
 #############################################################################
-#  &WritableDirAudit;
+#  &WriteableDirAudit;
 #    This subroutine uses a perl find to generate a list of all of the
 #    world writable directories for the target machine.  It then processes
 #    the these permissions and bases on directory name and nesting it
@@ -438,8 +393,8 @@ sub SUIDAudit {
 #
 #    uses B_TODO B_append_line B_create_file B_blank_file
 #############################################################################
-sub WritableDirAudit {
-    &ActionLog("# sub WritableDirAudit\n");
+sub WriteableDirAudit {
+    &ActionLog("# sub WriteableDirAudit\n");
     if (&getGlobalConfig("FilePermissions","world_writeable") eq "Y") {
 # Set the variable $File::Find::dont_use_nlink if you're using AFS,
 # since AFS cheats.
@@ -451,33 +406,109 @@ sub WritableDirAudit {
     *prune  = *File::Find::prune;
     my $strShell = ""; # Keeps all of the directories found by find.
 
-    &B_TODO("The directories listed in the shell script are world-writable.\n" .
-            "With the exception of /tmp and /var/tmp, world-writable \n" .
-            "directories are vulnerabilities.  You should examine \n" .
-            "each of the directories in the sh script to see if you want to \n" .
-            "leave it this way.  You should chmod +t <dirname> in \n" .
-            "most cases to ensure the default owner is the one who made \n" .
-            "the file.\n" .
-	    "The output has been formated in a shell script.\n" .
-	    "The shell script is can be found in the following location:\n" . 
+    &B_TODO("\n---------------------------------\nWriteable Directory Audit:\n" .
+	    "---------------------------------\n".
+            "Bastille has created a shell script to change the permissions\n" .
+            "on the world-writeable directories that it found.\n" .
+	    "The shell script can be found in the following location:\n" . 
 	    &getGlobal('BFILE',"directory-perms.sh") . "\n" . 
-	    "The values in this shell script are just suggestions\n" . 
-	    "and should be modified as you see fit.\n" . 
-	    "Note: Do not run the shell script without examining\n" . 
-	    "it closely. It will break some applications and therefore\n" . 
-	    "should be customized to your specific needs.\n\n");
+            "The script will add the \"sticky bit\" to all of the system's\n" . 
+	    "world writeable directories.  This will help mitigate the\n" . 
+	    "fact that directories often have to remain writeable.\n" .
+            "The script will also update the internal products database \n".
+	    "using the \"swmodify\" command and create another script \n".
+	    "which you can use to revert the actions taken if\n" .
+            "you need to.\n\n".
+
+	    "World-writeable directories are potential vulnerabilities.\n".
+	    "Examine each directories\' permissions in the shell script\n".
+	    "to see if you want to leave them this way.  The permissions\n".
+	    "in the script are just suggestions and should be modified to\n".
+	    "meet your system\'s needs.\n\n" . 
+
+	    "NOTE:    Do not run this UNSUPPORTED shell script without examining\n" . 
+	    "         it closely.  It might break some applications and\n" .
+            "         therefore should be customized to your specific needs.\n\n");
 
     &B_create_file(&getGlobal('BFILE', "directory-perms.sh"));
     # the unmatchable regexp will insure that the file is always 
     # blanked "a$b" or 'a' endofstring 'b'
     &B_blank_file(&getGlobal('BFILE', "directory-perms.sh"),'a$b');
+
+my $instructions = <<EOF;
+# Please review the following shell script and modify it to suit the 
+# needs of the system.  
+#
+# If you comment out a chmod, also comment out the echo
+# one line below the chmod, as the echo defines the
+# action to revert the chmod above.
+#
+# Here is a partial list of breakages to look out for if you use this script:
+# 
+#  - /tmp and /var/tmp sticky bit: applications which rely on unique process
+# id's in /tmp when run by different users may break when the process id's
+# are recycled (cleaning tmp directories regularly may alleviate this
+# problem)
+#
+#  - Log directories (most of which are named with the word "log" in them): 
+# Programs which are run by different users but create and/or write logs in
+# a common directory may fail to log actions.  This includes GUI error logs
+# in some versions of HP-UX diagnostic tools.
+#
+#  - "cat" directories such as those in /usr/share/man are used by the
+# "man" command to write pre-processed man pages.  Eliminating the
+# world-writeable bit will cause a degradation in performance because
+# the man page will have to be reformatted every time it is accessed.
+#
+#  - Some directories may have incorrect owners and/or groups.  Eliminating
+# world-writeable permisions on these directories have no effect if the
+# owner/group is set properly.  For example, one problem with HP Openview
+# running without world-writeable directories was corrected by the following:
+#
+#    /usr/bin/chown root:sys /var/opt/OV/analysis/ovrequestd/config
+#
+# This change has not been fully tested, but was shown to work when tested
+# in a limited, single-purpose environment.
+#
+#  - Change the directory /var/obam/translated may have an impact on non-root
+# users viewing help in obam (the GUI library used by swinstall, SAM,
+# older versions of ServiceControl Manager, and others)
+#
+#  - Eliminating the world-writeable permissions on socket directories has been
+# shown to stop the X server from operating properly.  However, setting the 
+# sticky bit instead (what this script will do by default) did not have the 
+# same effects.
+#
+#  - There are several other directories which have world-writeable permissions.
+# Some of these are shipped with HP-UX, others are shipped with 3rd party
+# products, and others may have been created by users without an appropriate
+# umask set.  Bastille will help you find those directories so that you can
+# make appropriate decisions for your environment.  The full impact of making
+# these changes has not been analyzed.
+#
+# As you run the script, it will create a "revert-directory-perms.sh"
+# script which will allow you to revert to a supported state (independent of
+# the rest of the HP-UX Bastille configurations, which are supported). 
+#
+# Because of the potential for very subtle breakages, you should also keep
+# a record of any changes which you make manually to your system so that
+# you can revert them to help debug any problems which you run into.
+# Running 'bastille -r' will revert all bastille changes, including
+# running the revert-directory-perms.sh script, but it may not revert
+# changes you have made manually.
+#
+# Comment out the following line if you accept that these changes are not
+# supported by HP, you have customized the rest of the script for your
+# needs, and you wish to make the changes to your system.
+echo "The changes made by this script are NOT supported by HP.\nChanging the permissions of directories in this way has the potential to break\ncompatibility with some applications and requires testing in your environment.\nTherefore, the script must be customized before running.\nExiting script without taking any action."; exit 1
+#
+EOF
+
     &B_append_line(&getGlobal('BFILE', "directory-perms.sh"),"0",
-		   "# Please review the following shell script and modify\n" . 
-		   "# it to suit the needs of the system.\n" .
-		   "# If you comment out a chmod also comment out the echo\n" .
-		   "# one line below it\n\n");
-    
-    
+                   $instructions .
+                   "# Location of the revert directory permissions script (to be run by \n" .
+                   "# Bastille's revert.)\n" .
+                   "REVERTPERMS=" . &getGlobal('BFILE',"revert-directory-perms.sh") . "\n\n");
 
 
     my @rwx = qw(--- --x -w- -wx r-- r-x rw- rwx);
@@ -500,7 +531,7 @@ sub WritableDirAudit {
 	    ! ($dev >= 0) &&
 		($File::Find::prune = 1)
 		    ||
-			(($mode & 02) == 02) && -d _ && &ls;
+			((($mode & 01000) == 00000) && (($mode & 02) == 02)) && -d _ && &ls;
     }
 
 sub sizemm {
@@ -519,38 +550,25 @@ sub ls {
     my $oldperm = "";
     my $perms = "";
 
-    if($pname !~ /tmp/){
-	$newperm = ($mode & 7) - 2;
-	$oldperm = ($mode & 7);
-	$perms = $rwx[$mode & 7];
-	$mode >>= 3;
+    $newperm = ($mode & 7);
+    $oldperm = ($mode & 7);
+    $perms = $rwx[$mode & 7];
+    $mode >>= 3;
+    
+    $newperm = ($mode & 7) . $newperm;
+    $oldperm = ($mode & 7) . $oldperm;
+    $perms = $rwx[$mode & 7] . $perms;
+    $mode >>= 3;
+    
+    $newperm = ($mode & 7) . $newperm;
+    $oldperm = ($mode & 7) . $oldperm;
+    $perms = $rwx[$mode & 7] . $perms;
+    $mode >>= 3;
+    
+    $newperm = (($mode & 7)+1) . $newperm;
+    $oldperm = ($mode & 7) . $oldperm;
 
-	$newperm = ($mode & 7) . $newperm;
-	$oldperm = ($mode & 7) . $oldperm;
-	$perms = $rwx[$mode & 7] . $perms;
-	$mode >>= 3;
-	
-	$oldperm = ($mode & 7) . $oldperm;
-	$newperm = ($mode & 7) . $newperm;
-	$perms = $rwx[$mode & 7] . $perms;
-    }
-    else {
-	$newperm = ($mode & 7);
-	$oldperm = ($mode & 7);
-	$perms = $rwx[$mode & 7];
-	$mode >>= 3;
-
-	$newperm = ($mode & 7) . $newperm;
-	$oldperm = ($mode & 7) . $oldperm;
-	$perms = $rwx[$mode & 7] . $perms;
-	$mode >>= 3;
-	
-	$newperm = ($mode & 7) . $newperm;
-	$oldperm = ($mode & 7) . $oldperm;
-	$perms = $rwx[$mode & 7] . $perms;
-	$newperm = "1" . $newperm;
-	$oldperm = "0" . $oldperm;
-    }
+    
     substr($perms, 2, 1) =~ tr/-x/Ss/ if -u _;
     substr($perms, 5, 1) =~ tr/-x/Ss/ if -g _;
     substr($perms, 8, 1) =~ tr/-x/Tt/ if -k _;
@@ -572,71 +590,68 @@ sub ls {
         $timeyear = sprintf("%02d:%02d", $hour, $min);
     }
     my $swString = "";
+    my $shSafePname = $pname;
+ 
+    $shSafePname =~ s/([\"\`\$\\])/\\$1/g;
+    # The IPD allows directories to be defined as /dir or /dir/ 
+    # so a check for either is appropriate.
     if(defined &getGlobalSwlist("$pname")){
-	# defining additional swmodify command if directory is in IPD. 
-	$swString = "; " . &getGlobal('BIN',"swmodify") . 
-	    " -x files=\'$pname\' " . &getGlobalSwlist("$pname");
-    } 
+	# defining additional swmodify command if directory is in IPD.
+	my @products = @{ &getGlobalSwlist("$pname")};
+	foreach my $product (@products) {
+	    $swString .= " ; " . &getGlobal('BIN',"swmodify") . " -x files=\"${shSafePname}\" " . $product;
+	}
+    } # dangleing / case
+    elsif(defined &getGlobalSwlist("${pname}\/")) {
+        # defining additional swmodify command if directory is in IPD.
+	my @products = @{ &getGlobalSwlist("${pname}\/")};
+	foreach my $product (@products) {
+	    $swString .= " ; " . &getGlobal('BIN',"swmodify") . " -x files=\"${shSafePname}\/\" " . $product;
+	}
+    }
+   
     # Generating WORLD WRITEABLE DIR shell commands for directory-perms.sh
+    my $echoSafeRevert = &getGlobal('BIN', "chmod") . " $oldperm \"$shSafePname\"" . $swString;
+
+    # if our distro matches HP-UX then we need a layer of escapes for System 5 echo
+    if( &GetDistro =~ "^HP-UX") {
+	$echoSafeRevert =~ s/([\\])/\\$1/g;
+    }
+
+    # add second level of escaping for posix shell
+    $echoSafeRevert =~ s/([\"\`\$\\])/\\$1/g;
+
     $strShell .= "# $perms   $user   $group   $moname[$mon]   $mday   " . 
-            "$timeyear   $pname \n" . 
-	     &getGlobal('BIN', "chmod") . " $newperm \'$pname\'" . $swString .
-	     "\n" . &getGlobal('BIN',"echo") . " \"" .
-	     &getGlobal('BIN', "chmod") . " $oldperm \'$pname\'" . $swString .
-	     "\" >> " . &getGlobal('BFILE',"undo-directory-perms.sh") . "\n";
-			;
+            "$timeyear   $shSafePname \n" . 
+	     &getGlobal('BIN', "chmod") . " $newperm \"$shSafePname\"" . $swString .
+	     "\n" . "echo" . " \"$echoSafeRevert\" >> " .  '$REVERTPERMS' . "\n\n";
+    
 } # ls subroutine
 
-# This section prepends undo-directory-perms.sh to the undo-actions file.
-# undo-directory-perms is generated by directory-perms.sh and unmakes the
+# This section prepends revert-directory-perms.sh to the revert-actions file.
+# revert-directory-perms is generated by directory-perms.sh and unmakes the
 # script's permission changes.
-$strShell .= &getGlobal('BIN',"mv") . " " . &getGlobal('BFILE',"undo-actions") .
-    " " . &getGlobal('BFILE',"undo-actions") . ".bastille\n" .
-    &getGlobal('BIN',"echo") . " \"" . &getGlobal('BFILE',"undo-directory-perms.sh") .
-    "\" > " .  &getGlobal('BFILE',"undo-actions") . "\n" . 
+$strShell .= "# Editing Bastille's revert scripts to include permissions changes\n# PLEASE DO NOT EDIT BELOW\n" . 
+    &getGlobal('BIN',"mv") . " " . &getGlobal('BFILE',"revert-actions") .
+    " " . &getGlobal('BFILE',"revert-actions") . ".bastille\n" .
+    &getGlobal('BIN',"echo") . " \"" . &getGlobal('BFILE',"revert-directory-perms.sh") .
+    "\" > " .  &getGlobal('BFILE',"revert-actions") . "\n" . 
     &getGlobal('BIN',"echo") . " \"" . &getGlobal('BIN',"mv") . " " .
-    &getGlobal('BFILE',"undo-directory-perms.sh") . " " . &getGlobal('BFILE',"undo-directory-perms.sh") .
-    ".last\" >> " .  &getGlobal('BFILE',"undo-actions") . "\n" . 
+    &getGlobal('BFILE',"revert-directory-perms.sh") . " " . &getGlobal('BFILE',"revert-directory-perms.sh") .
+    ".last\" >> " .  &getGlobal('BFILE',"revert-actions") . "\n" . 
     &getGlobal('BIN',"echo") . " \"" . &getGlobal('BIN',"chmod") . " 500 " .
-    &getGlobal('BFILE',"undo-directory-perms.sh") . ".last\" >> " .  
-    &getGlobal('BFILE',"undo-actions") . "\n" . 
-    &getGlobal('BIN',"cat") . " " .  &getGlobal('BFILE',"undo-actions") . ".bastille" .
-    " >>" .  &getGlobal('BFILE',"undo-actions") . "\n" .
-    &getGlobal('BIN',"rm") . " " .  &getGlobal('BFILE',"undo-actions") . ".bastille\n" .
-    &getGlobal('BIN',"chown") . " root:sys " .  &getGlobal('BFILE',"undo-actions") . "\n" .
-    &getGlobal('BIN',"chmod") . " 0500 " .  &getGlobal('BFILE',"undo-actions") . "\n" .
-    &getGlobal('BIN',"chown") . " root:sys " .  &getGlobal('BFILE',"undo-directory-perms.sh") . "\n" .
-    &getGlobal('BIN',"chmod") . " 0500 " .  &getGlobal('BFILE',"undo-directory-perms.sh") . "\n";
-&B_append_line(&getGlobal('BFILE', "directory-perms.sh"),"0",$strShell);
+    &getGlobal('BFILE',"revert-directory-perms.sh") . ".last\" >> " .  
+    &getGlobal('BFILE',"revert-actions") . "\n" . 
+    &getGlobal('BIN',"cat") . " " .  &getGlobal('BFILE',"revert-actions") . ".bastille" .
+    " >>" .  &getGlobal('BFILE',"revert-actions") . "\n" .
+    &getGlobal('BIN',"rm") . " " .  &getGlobal('BFILE',"revert-actions") . ".bastille\n" .
+    &getGlobal('BIN',"chown") . " root:sys " .  &getGlobal('BFILE',"revert-actions") . "\n" .
+    &getGlobal('BIN',"chmod") . " 0500 " .  &getGlobal('BFILE',"revert-actions") . "\n" .
+    &getGlobal('BIN',"chown") . " root:sys " .  &getGlobal('BFILE',"revert-directory-perms.sh") . "\n" .
+    &getGlobal('BIN',"chmod") . " 0500 " .  &getGlobal('BFILE',"revert-directory-perms.sh") . "\n";
+   &B_append_line(&getGlobal('BFILE', "directory-perms.sh"),"0",$strShell);
 
-} # if question is yes
+   } # if question is yes
 } # End Audit Subroutine
+
 1;
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
