@@ -17,7 +17,7 @@
 # changes.
 
 ##########################################
-## Bastille Linux Text User Interface   ##
+## Bastille Text User Interface         ##
 ##########################################
 
 #
@@ -50,13 +50,6 @@ sub do_Bastille {
     $detail_button{"low"}="< Explain More >";
     $detail_button{"high"}="< Explain Less >";
 
-    # Load in the questions database
-    $Number_Modules=&Load_Questions;
-    #&Load_Questions_Stub;
-
-    # Check for an existing config file and populate the default answers from it
-    &Populate_Answers;
-
     # Set up the Curses environment
     $window = new Curses;
 
@@ -70,7 +63,8 @@ sub do_Bastille {
         print "\nand can return to your graphical display, usually, by hitting\n";
         print "\n            <CTRL>-<ALT>-<F7>";
         print "\n\n**************************************************************\n";
-        die "Not in 80 columns...";
+        &ErrorLog("ERROR:   Not in 80 columns...\n");
+        exit(1);
     }
 
     start_color;
@@ -115,7 +109,7 @@ sub do_Bastille {
     	   'content'      => $credits,
     	   'border'       => 'blue',
     	   'draw_only'    => 0,
-    	   'title'        => "Bastille Linux Credits     (press TAB to go on)",
+    	   'title'        => "Bastille Credits     (press TAB to go on)",
     	   'pos'          => 1,
     	   'regex'	  => "\n\t"
     	   );
@@ -372,11 +366,13 @@ sub Redraw_Screen {
 sub Draw_Title {
 # Draw the title bar   
 
+    my $windowname = "Bastille";
+
     txt_field( 'window' => $window, 
 	       'ypos'   => 0, 'xpos'   => 0,
 	       'lines'  => 1,
 	       'cols'   => $COLS-2, 
-	       'content'=> "                                Bastille Linux",
+	       'content'=> "                                  $windowname",
 	       'border' => 'blue', 
 	       'draw_only' => 1,
 	       );
