@@ -1,6 +1,16 @@
 #!/usr/bin/perl
 
-# This is a Tk interface for Bastille-Linux.  It is derived from
+# Copyright (C) 2000 Jay Beale
+# Licensed under the GNU General Public License    
+#
+# Tk additions copyright (C) 2000 by Paul L. Allen and licensed
+# under the GNU General Public License.
+#
+# Additional changes, corrections, and feature enhancements are
+# copyright (C) 2001-2003 by Hewlett Packard Company and licensed
+# under the GNU General Public License.
+
+# This is a Tk interface for Bastille.  It is derived from
 # the Bastille-1.1.1 InteractiveBastille.pl script, with the logic
 # warped into the events-and-callbacks programming model typical
 # of X programming.  It requires the Tk.pm perl module and at 
@@ -40,11 +50,6 @@
 # The portions of this file derived from the original Bastille 1.1.1
 # InteractiveBastille.pl script retain this copyright notice:
 #
-# Copyright (C) 2000 Jay Beale
-# Licensed under the GNU General Public License    
-#
-# Tk additions copyright (C) 2000 by Paul L. Allen and licensed
-# under the GNU General Public License.
 #
 
 #use Data::Dumper;
@@ -60,13 +65,16 @@
 #
 #    http://www.perl.com/CPAN-local/modules/by-module/Tk/Tk800.022.tar.gz
 #
-if ( -d "/usr/lib/perl5/site_perl/5.6.0/i386-linux" ) {
-	use lib "/usr/lib/perl5/site_perl/5.6.0/i386-linux";
-}
+# This was already done in InteractiveBastille, so don't do it here anymore
+#if ( -d "/usr/lib/perl5/site_perl/5.6.0/i386-linux" ) {
+#	use lib "/usr/lib/perl5/site_perl/5.6.0/i386-linux";
+#}
 
 use Tk;
 use Tk::LabFrame;
 use Tk::DialogBox;
+use Tk::HList;
+
 
 sub do_Bastille {
 
@@ -84,61 +92,16 @@ sub do_Bastille {
 	$detail_button{"low"}="Explain More";
 	$detail_button{"high"}="Explain Less";
 
-#	Load in the questions database
-#
-	$Number_Modules=&Load_Questions;
-
-#	Replace the title screen with appropriate verbiage and give it
-#	a module name so it'll behave consistently in the module listbox.
-#
-	$Question{"Title_Screen"}{"short_exp"}=<<END_TITLESCREEN;
-                             (Tk User Interface)
- 
-                                   v1.2.0
- 
- 
-       Please answer all the questions to build a more secure system.
- 
-       The Next and Back buttons navigate forward and backward in the 
-       questions database.  Changes made in the Answer field are *only*
-       saved when you push the Next button!  The "modules" in the
-       questions database are listed to the left.  You can jump to
-       the start of any module simply by clicking on its name.
-
-       Some questions have two levels of explanatory text, which you
-       can adjust with the Explain Less/More button. 
-
-       Please address bug reports and suggestions to jay\@bastille-linux.org
-       Bugs in the Tk user interface are the fault of allenp\@nwlink.com.
-END_TITLESCREEN
-	$Question{"Title_Screen"}{'module'} = "Title Screen";
-
-#	Adjust the no_epilogue text on the End Screen and give it a module name
-#	for consistency.
-#
-    $Question{"End_Screen"}{"no_epilogue"}= <<END_BACK_NEXT;
-Please choose a module from the list at left, and use Back/Next buttons 
-to move among the questions you wish to change.
-
-
-      Choose YES on this question later to implement your choices.
-END_BACK_NEXT
-	$Question{"End_Screen"}{'module'} = "End";
-
-#	If there is an existing config file, snarf all the answers.
-#
-	&Populate_Answers;
-
 	$next_question_index="Title_Screen";
 
 	&Ask($next_question_index);
 	$first_time = 0;
 
-	MainLoop;
+	&MainLoop;
 
 }
 
-sub Ask {
+sub Ask($) {
 # sub Ask (index to Question{} record)
 
 ##############################################################################
@@ -164,8 +127,10 @@ sub Ask {
     $in_epilogue = 0;
 
     my $frameopts  = ['-side' => 'top', '-padx' => 5, '-pady' => 5, '-ipadx' => 5, '-ipady' => 5, '-fill' => 'x'];
+    my $eframeopts  = ['-side' => 'top', '-padx' => 5, '-pady' => 5, '-ipadx' => 5, '-ipady' => 5, '-fill' => 'both',
+		       '-expand' => 1];
     my $lframeopts  = ['-side' => 'left', '-padx' => 5, '-pady' => 5, '-ipadx' => 5, '-ipady' => 5, '-fill' => 'y'];
-
+    my $hlistopts  = ['-side' => 'left', '-padx' => 0, '-pady' => 0, '-ipadx' => 0, '-ipady' => 0, '-fill' => 'y'];
 #
 #	Load question into local variables
 #
@@ -180,7 +145,7 @@ sub Ask {
 #	    print Dumper($Question{$index}) . "\n";
 	    $index = $Question{$index}{"yes_child"};
     }
-
+    
     my $short_exp      =$Question{$index}{"short_exp"};
     my $long_exp       =$Question{$index}{"long_exp"};
     my $question       =$Question{$index}{"question"};
@@ -193,13 +158,14 @@ sub Ask {
     my $proper_parent  =$Question{$index}{"proper_parent"};
     my $answer     =$Question{$index}{"answer"};
     my $module     =$Question{$index}{"module"};
+    ($shortModule, $rest) = split(/\.pm/, $module); 
     
     if ($first_time) {
 #
 #	The main window
 #
 	$mw = MainWindow->new();
-	$mw->title("Bastille Linux");
+        $mw->title("Bastille");
 
 #	Frames to hold the modules listbox, question, explanation,
 #	answer, and buttons.
@@ -212,7 +178,7 @@ sub Ask {
 	    '-labelside' => "acrosstop")->pack(@$frameopts);
 	$eframe = $mw->LabFrame(
 	    '-label' => "Explanation", 
-	    '-labelside' => "acrosstop")->pack(@$frameopts);
+	    '-labelside' => "acrosstop")->pack(@$eframeopts);
 	$aframe = $mw->LabFrame(
 	    '-label' => "Answer", 
 	    '-labelside' => "acrosstop")->pack(@$frameopts);
@@ -221,24 +187,37 @@ sub Ask {
 
 #	A scrolled listbox to hold module names
 #
-	$list = $lframe->Scrolled('Listbox',
-	    '-scrollbars' => 'e',
-	    '-width' => 20,
-	    '-height' => 20)->pack();
-	$list->bind("<1>", \&listbox_callback);
+#	$list = $lframe->Scrolled('Listbox',
+#	    '-scrollbars' => 'e',
+#	    '-width' => 20,
+#	    '-selectmode' => 'single',
+
+
+
+	$list = $lframe->Scrolled( 'HList',
+				   '-drawbranch'    => 0,
+				   '-scrollbars' => 'e',
+				   '-width'         => 25,
+				   '-indent'        => 5,
+				   '-selectmode'    => 'single')->pack(@$hlistopts);
+	$list->bind("<ButtonRelease-1>", \&hlist_callback);
+	$list->bind("<ButtonPress-1>", \&hlist_callback);
+
+	$list->pack();
+	
+	$completeBitmap = $lframe->Bitmap('-file' => &getGlobal('BFILE',"complete.xbm"));
+	$incompleteBitmap = $lframe->Bitmap('-file' => &getGlobal('BFILE',"incomplete.xbm"));
 
 #	The question
 #
 	$qe  = $qframe->Entry(
-	    '-width' => 80)->pack();
+	    '-width' => 80)->pack('-fill' => 'x');
 
 #	A scrolled Text widget for the explanation
 #
 	$tw = $eframe->Scrolled('Text',
 	    '-wrap' => 'word',
-	    '-scrollbars' => 'e',
-	    '-width' => 80,
-	    '-height' => 20)->pack();
+	    '-scrollbars' => 'e')->pack('-fill' => 'both','-expand' => 1);
 
 #	The answer.  Note that there are three widgets defined here,
 #	but their pack() methods have not been called.  This allows
@@ -260,17 +239,18 @@ sub Ask {
 	    '-text' => "No",
 	    '-value' => "N",
 	    '-variable' => \$answer_text);
+	$spacer = $aframe->Label('-width' => 5);
 
 #	The Next, Back, and Detail buttons
 #
-	$bframe->Button(
-	    '-text' => "Back",
+	$backButton = $bframe->Button(
+	    '-text' => "<- Back",
 	    '-command' => \&back_button)->pack(
 	    '-expand' => 1,
 	    '-side' => 'left');
-	$bframe->Button(
-	    '-text' => "Next",
-	    '-command' => \&next_button)->pack(
+	$defaultButton = $bframe->Button(
+	    '-text' => "Restore Default",
+	    '-command' => \&default_button)->pack(
 	    '-expand' => 1,
 	    '-side' => 'left');
 	$detail_toggle = "high";
@@ -279,6 +259,12 @@ sub Ask {
 	    '-expand' => 1,
 	    '-side' => 'left');
 
+	$okButton = $bframe->Button(
+	    '-text' => "OK ->",
+	    '-command' => \&next_button)->pack(
+	    '-expand' => 1,
+	    '-side' => 'left');
+
 #	Build the list of modules.  The @module_index array
 #	translates a module number into the first index for
 #	that module in the %Question hash.  This allows us to
@@ -294,8 +280,20 @@ sub Ask {
 	    if ($mod ne $Question{$nq}{'module'}) {
 	    	unless ($nq eq "End_Screen") {
 	    		$mod = $Question{$nq}{'module'};
-	    		($shortmod, $rest) = split(/.pm/, $mod);
-	    		$list->insert($ndx, $shortmod);
+	    		my ($shortmod, $rest) = split(/.pm/, $mod);
+			
+			my ($incompleteModule, $incompleteKey) = &checkQtree($nq);
+	
+			if($incompleteModule eq $shortmod) {
+			    $list->add($ndx, '-itemtype' => 'imagetext',
+				       '-image' => $incompleteBitmap,
+				       '-text' => $shortmod);
+			}
+			else {
+			    $list->add($ndx, '-itemtype' => 'imagetext',
+				       '-image' => $completeBitmap,
+				       '-text' => $shortmod);
+			}
 	    		$module_index[$ndx]{'index'} = $nq;
 	    		$reverse_module_index{$shortmod} = $ndx;
 	    		$module_index[$ndx]{'done'} = 0;
@@ -304,19 +302,78 @@ sub Ask {
 	    }
 	    $nq = $Question{$nq}{'yes_child'};
 	}
-#
+
 #	Fudge things so the end screen gets indexed properly
 #
-	$list->insert($ndx, "End Screen");
+
+	my ($incompleteModule, $incompleteKey) = &checkQtree($first_question);
+	if($incompleteModule ne "") {
+	    $list->add($ndx, '-itemtype' => 'imagetext',
+		       '-image' => $incompleteBitmap,
+		       '-text' => "End Screen");
+	}
+	else {
+	    $list->add($ndx, '-itemtype' => 'imagetext',
+		       '-image' => $completeBitmap,
+		       '-text' => "End Screen");
+	}
+
 	$module_index[$ndx]{'index'} = "End_Screen";
 	$reverse_module_index{'End'} = $ndx;
 	$module_index[$ndx]{'done'} = 0;
+
+
+#  Setup Routine for Hlist
+	
     }
+    # may be able to do this cleanly...
+    for(my $i=0; $i < $#module_index; $i++){
+	my $currentKey = $module_index[$i]{'index'};
+	my $currentMod = $Question{$currentKey}{'module'};
+	my ($currentShortMod, $trash) = split /.pm/, $currentMod;
+	my ($incompleteModule, $incompleteKey) = &checkQtree($module_index[$i]{'index'});
+	if($incompleteModule eq $currentShortMod) {
+	    $list->entryconfigure($i, '-image' => $incompleteBitmap);
+	}
+	else {
+	    $list->entryconfigure($i, '-image' => $completeBitmap);
+	    #module pruner
+
+	}
+    }
+
+
+    
+    # Button controls
+    if($Question{$index}{'proper_parent'} eq $index){
+	$backButton->configure('-state' => 'disabled');
+    }
+    else {
+	$backButton->configure('-state' => 'normal');
+    }
+
+
+    if(! defined $Question{$index}{'default_answer'}){
+	$defaultButton->configure('-state' => 'disabled');
+    }
+    else {
+	$defaultButton->configure('-state' => 'normal');
+    }
+
+
+    if(! defined $Question{$index}{'long_exp'}){
+	$db->configure('-state' => 'disabled');
+    }
+    else {
+	$db->configure('-state' => 'normal');
+    }
+	
 
     ($modulename, $rest)  = split(/.pm/, $module);
     $moduleindex = $index;
     $db->configure('-text' => $detail_button{$detail_toggle});
-    $list->selectionClear('0', 'end');
+    $list->selectionClear('0', $reverse_module_index{"End"});
+    $list->anchorClear;
     unless ($modulename eq "") {
     	$list->selectionSet($reverse_module_index{$modulename});
     }
@@ -339,17 +396,21 @@ sub Ask {
 
 #	Now, clear the screen fields and insert the new values.
 #
+    $tw->configure('-state' => 'normal');
     $tw->delete('0.0', 'end');
     $tw->insert('0.0', $explanation);
+    $tw->configure('-state' => 'disabled');
 
 #	Trim the default answer from the end of the question, since
 #	it might differ from the one we found in the config file.
 #
     $question =~ s/\[.*\]$//;
 
+    $qe->configure('-state' => 'normal');
     $qe->delete('0.0', 'end');
     $qe->insert('0.0', $question);
-    
+    $qe->configure('-state' => 'disabled');
+   
     $answer_text = $answer;
     if ($question eq "") {
 #
@@ -364,11 +425,13 @@ sub Ask {
 #
 	if ($toggle_yn) {
 	    $ae->pack('forget');
-	    $ab1->pack('-side' => 'left');
-	    $ab2->pack('-side' => 'left');
+	    $spacer ->pack('-side' => 'right');
+	    $ab1->pack('-side' => 'right');
+	    $ab2->pack('-side' => 'right');
 	} else {
 	    $ab1->pack('forget');
 	    $ab2->pack('forget');
+	    $spacer ->pack('forget');
 	    $ae->pack();
 	}
     }
@@ -378,36 +441,56 @@ sub Ask {
 #
 sub next_button {
     my $answer  = $answer_text;
-
     unless ($in_epilogue) {
 	if ($Question{$index}{'toggle_yn'}) {
 	    if ($answer =~ /[Yy]/)  {
 		$Question{$index}{'answer'} = "Y";
+		$GLOBAL_CONFIG{$shortModule}{$index} = "Y";
 		$next_index = $Question{$index}{'yes_child'};
+		$Question{$next_index}{'proper_parent'} = $index;
 		if ($Question{$index}{'yes_epilogue'} and not $in_epilogue) {
 		    $in_epilogue = 1;
 		    &show_epilogue ("yes_epilogue");
 		    return;
 		}
-	    } elsif ($answer =~ /[Nn]/)  {
+	    } 
+	    elsif ($answer =~ /[Nn]/)  {
 		$Question{$index}{'answer'} = "N";
+		$GLOBAL_CONFIG{$shortModule}{$index} = "N";
 		$next_index = $Question{$index}{'no_child'};
+
+		if($index ne "End_Screen") {
+		    $Question{$next_index}{'proper_parent'} = $index;
+		}
+
 		if ($Question{$index}{'no_epilogue'} and not $in_epilogue) {
 		    $in_epilogue = 1;
 		    &show_epilogue ("no_epilogue");
 		    return;
 		}
-	    } else {
+	    } 
+	    else {
 		$mw->bell();
 		return;
 	    }
-	} else {
-	    $Question{$index}{'answer'} = $answer;
-	    $next_index = $Question{$index}{'yes_child'};
-	    if ($Question{$index}{'yes_epilogue'} and not $in_epilogue) {
-		$in_epilogue = 1;
-		&show_epilogue ("yes_epilogue");
-		return;
+	} 
+	else {
+	    if(&validateAnswer($index,$answer)) {
+		$Question{$index}{'answer'} = $answer;
+		$GLOBAL_CONFIG{$shortModule}{$index} = $answer;
+		$next_index = $Question{$index}{'yes_child'};
+		$Question{$next_index}{'proper_parent'} = $index;
+		if ($Question{$index}{'yes_epilogue'} and not $in_epilogue) {
+		    $in_epilogue = 1;
+		    &show_epilogue ("yes_epilogue");
+		    return;
+		}
+	    }
+	    else {
+		$Question{$index}{'answer'} = $answer;
+		delete $GLOBAL_CONFIG{$shortModule}{$index};
+		&show_validateError;
+		$next_index = $index;
 	    }
 	}
     }
@@ -415,8 +498,16 @@ sub next_button {
 #    Show the credits if we've reached the end.
 #
     if ($next_index eq "RUN_SCRIPT") {
-	&finish_up;
-	return;
+
+	my ($incompleteModule, $incompleteKey) = &checkQtree($first_question);
+	
+	if($incompleteModule eq "") {
+	    &promptSave;  # save changes made to config?
+	    return;
+	}
+	else {
+	    $next_index = &notFinished($incompleteModule,$incompleteKey);
+	}
     }
 
     &Ask($next_index);
@@ -431,10 +522,19 @@ sub show_epilogue {
     $ae->pack('forget');
     $ab1->pack('forget');
     $ab2->pack('forget');
+
+    $qe->configure('-state' => 'normal');
+    $tw->configure('-state' => 'normal');
     $qe->delete('0.0', 'end');
     $answer_text = "";
     $tw->delete('0.0', 'end');
     $tw->insert('0.0', $Question{$index}{$field});
+    $qe->configure('-state' => 'disabled');
+    $tw->configure('-state' => 'disabled');
+
+    $defaultButton->configure('-state' => 'disabled');
+    $db->configure('-state' => 'disabled');
+
 }
 
 #	Display the credits and wait for the user to say OK
@@ -442,25 +542,99 @@ sub show_epilogue {
 sub finish_up {
     my @tw_opts = ('-padx' => 5, '-pady' => 5, '-side' => 'top');
     my @but_opts = ('-padx' => 5, '-pady' => 5, '-side' => 'left', '-expand' => 1);
-    &Output_Config_Files;
     $finishwin = $mw->Toplevel;
     $finishwin->title("Finishing Up");
     $finishwin->Label(
         '-text' => "Configuration file has been saved.\n\nWhat would you like to do now?")->pack(@tw_opts);
-    $finishwin->Button(
-        '-text' => "Apply Config to System",
-        '-command' => \&apply_config)->pack(@but_opts);
-    $finishwin->Button(
-        '-text' => "Go Back and Change Config",
-        '-command' => \&go_back)->pack(@but_opts);
     $finishwin->Button(
         '-text' => "Exit Without Changing System",
         '-command' => \&show_credits)->pack(@but_opts);
+    $finishwin->Button(
+        '-text' => "Go Back and Change Configuration",
+        '-command' => [sub {$mw->deiconify;&go_back;}])->pack(@but_opts);
+   $finishwin->Button(
+        '-text' => "Apply Configuration to System",
+        '-command' => \&apply_config)->pack(@but_opts);
     $finishwin->grab;
 }
 
+###################################################################
+#  &notFinished($module,$key);                                    #
+#    This subroutine displays a window that tells the user that   #
+#    they have not finished answering all questions and will have #
+#    to before the backend will run.  Three options are given:    #
+#    Finish answering question, Exit without saving, and save and #
+#    exit.                                                        #
+#                                                                 #
+#   REQUIRES:  &Ask($key);                                        #
+#   REQUIRES:  &partialSave                                       #
+#                                                                 #
+###################################################################
+sub notFinished($$) {
+    my ($module,$key) = @_;
+    my @tw_opts = ('-padx' => 5, '-pady' => 5, '-side' => 'top');
+    my @but_opts = ('-padx' => 5, '-pady' => 5, '-side' => 'left', '-expand' => 1);
+
+    $unfinishedwin = $mw->Toplevel;
+    $unfinishedwin->title("Warning");
+    $unfinishedwin->Label(
+        '-text' => "You have not answered all of the questions that pertain to your system.\n" . 
+		   "In order for Bastille to apply changes you must answer all of the\n" . 
+		   "questions that are relevant to your system.\n")->pack(@tw_opts);
+    $unfinishedwin->Button(
+        '-text' => "Exit Without Saving",
+        '-command' => [sub { exit;}])->pack(@but_opts);
+    $unfinishedwin->Button(
+        '-text' => "Go Back and Finish Configuration",
+        '-command' => [sub {$mw->deiconify;&Ask($key); $unfinishedwin->destroy;}])->pack(@but_opts);
+    $unfinishedwin->Button(
+        '-text' => "Save and Exit",
+        '-command' => [sub { &partialSave; $unfinishedwin->destroy; exit;}])->pack(@but_opts);
+    $unfinishedwin->grab;
+
+}
+
+
+###################################################################
+#  &promptSave;                                                   #
+#    This subroutine displays a window that prompts the user to   #
+#    save the configuration file or exit without saving.          #
+#                                                                 #
+#   REQUIRES:  &Output_Config_Files;                              #
+#                                                                 #
+###################################################################
+sub promptSave {
+
+    my @tw_opts = ('-padx' => 5, '-pady' => 5, '-side' => 'top');
+    my @but_opts = ('-padx' => 5, '-pady' => 5, '-side' => 'left', '-expand' => 1);
+
+    $promptSaveWin = $mw->Toplevel;
+    $mw->withdraw;  # Hide the main window to help avoid confusion bewteen windows
+    $promptSaveWin->title("Save Configuration Changes?");
+    $promptSaveWin->Label(
+        '-text' => "Would you like to save the changes made to your Bastille configuration?\n" .
+		   "Saving configuration changes will not apply the configuration to your system.\n" .
+		   "If you do not save your configuration now, all changes made during this\n" . 
+		   "session will be lost.\n")->pack(@tw_opts);
+    $promptSaveWin->Button(
+        '-text' => "Exit Without Saving",
+        '-command' => [sub { exit; }])->pack(@but_opts);
+    $promptSaveWin->Button(
+        '-text' => "Go Back and Change Configuration",
+        '-command' => [sub { $mw->deiconify;$promptSaveWin->destroy;}])->pack(@but_opts);
+    $promptSaveWin->Button(
+        '-text' => "Save Configuration",
+        '-command' => [sub { &outputConfig; $promptSaveWin->destroy; &finish_up;}])->pack(@but_opts);
+    $promptSaveWin->grab;
+
+}    
+
 sub apply_config {
-    &Run_Bastille_with_Config;
+    if (fork) { #If parent, exec the backend, if child, continue handling Tk interface
+       exec &getGlobal('BFILE',"BastilleBackEnd");
+    }
+#   &Run_Bastille_with_Config; #commented out since now handling differently than Curses interface
+                               #(curses and Tk used to share this InteractiveBastille function)
     &show_credits;
 }
 
@@ -471,8 +645,9 @@ sub go_back {
 sub show_credits {
     my @tw_opts = ('-padx' => 5, '-pady' => 5, '-side' => 'top');
     $finishwin->destroy;
-    $credwin = $mw->Toplevel;
-    $credwin->title("Bastille Credits");
+    $mw->destroy;
+    $credwin = MainWindow->new();
+    $credwin->title("Credits");
     
     open CREDITS, $GLOBAL_BFILE{"credits"};
     @creditsarray = <CREDITS>;
@@ -483,22 +658,50 @@ sub show_credits {
         '-wrap' => 'word',
         '-height' => 24,
         '-width' => 80)->pack(@tw_opts)->insert('end', $creditstext);
-    $credwin->Button('-text' => "Exit", '-command' => [sub {exit;}])->pack(@tw_opts);
+    $credwin->Button('-text' => "Close", '-command' => [sub {exit;}])->pack(@tw_opts);
     $credwin->grab;
 }
 
 # This is the callback for the Back button
 #
 sub back_button {
+    my $answer  = $answer_text;
+    unless ($in_epilogue) {
+	if ($Question{$index}{'answer'} ne $answer){
+	    delete $GLOBAL_CONFIG{$shortModule}{$index};
+
+	    if ($Question{$index}{'toggle_yn'}) {
+		if ($answer =~ /[Yy]/)  {
+		    $Question{$index}{'answer'} = "Y";
+		} 
+	    elsif ($answer =~ /[Nn]/)  {
+		$Question{$index}{'answer'} = "N";
+	    } 
+	    } 
+	    else {
+		$Question{$index}{'answer'} = $answer;
+	    }
+	}
+    }
+
     if ($in_epilogue) {
         &Ask($index);
-    } else {
+    } 
+    else {
         &Ask($Question{$index}->{'proper_parent'});
     }
 }
 
+sub default_button {
+    my $answer = $Question{$index}{'default_answer'};
+    $Question{$index}{'answer'} = $answer;
+    delete $GLOBAL_CONFIG{$shortModule}{$index};
+    &Ask($index);
+}
+
 # This is the callback for the Details toggle button.
 #
+
 sub expn_button {
     my $explanation;
 
@@ -521,35 +724,60 @@ sub expn_button {
     } else  {
         $explanation = "";
     }
+    
+    $tw->configure('-state' => 'normal');
     $tw->delete('0.0', 'end');
     $tw->insert('0.0', $explanation);
-
+    $tw->configure('-state' => 'disabled');
 }
 
 # This is the listbox callback
 #
-sub listbox_callback {
-    my $sel = $list->curselection;
-
-    my @tw_opts = ('-padx' => 5, '-pady' => 5, '-side' => 'top');
-    my @but_opts = ('-padx' => 5, '-pady' => 5, '-side' => 'left', '-expand' => 1);
-
-    $warnwin = $mw->DialogBox;
-    $warnwin->title("Warning");
-    $warnwin->Label(
-        '-text' => "If you have made any modifications to the current module,\n" .
-                 "Please be sure to complete the module by clicking on \"Next\"\n" .
-                 "until you have finished the module before skipping to another \n" .
-                 "module.  If you don't, you might leave the answers in an \n" .
-                 "inconsistent state.")->pack(@tw_opts);
-    $warnwin->Button(
-        '-text' => "Continue anyway",
-        '-command' => [sub { &Ask ($module_index[$sel]{'index'}); $warnwin->destroy; }])->pack(@but_opts);
-    $warnwin->Button(
-        '-text' => "Cancel",
-        '-command' => [sub { $warnwin->destroy; $list->selectionClear('0', 'end'); }])->pack(@but_opts);
-    my $answer = $warnwin->Show();
-
+sub hlist_callback {
+    my $sel = $list->info(selection);
+    if($sel ne ""){
+	$list->selectionClear('0', $reverse_module_index{"End"});
+	$list->selectionSet($sel);
+	$list->anchorClear;
+	&Ask ($module_index[$sel]{'index'});
+    } 
 }
 
+
+sub show_validateError {
+
+    my $vRegExp = &getRegExp($index);
+    my $exampleString = "";
+    if(exists $Question{$index}{"expl_ans"} &&  $Question{$index}{"expl_ans"} ne "") {
+	my $example = $Question{$index}{"expl_ans"};
+	$exampleString = "An example of an acceptable answer is:\n\t$example\n\n";
+    }
+    elsif(exists $Question{$index}{"default_answer"} && $Question{$index}{"default_answer"} ne "") {
+	my $example = $Question{$index}{"default_answer"};
+	$exampleString = "An example of an acceptable answer is:\n\t$example\n\n";
+    }
+
+    my $vErrorWin = $mw->DialogBox;
+    $vErrorWin->title("Input Error:");
+    $vErrorWin->Label( '-text' =>
+			  "\nThis question requires an answer with a specific\n" .
+			  "format.\n\n" . $exampleString  .
+		          "See the question explanation for more details.",
+		       '-justify' => 'left' )->pack(@tw_opts);
+    $vErrorWin->Show();
+}
+
+
 1;
+
+
+
+
+
+
+
+
+
+
+
+
